#ifndef TINY_H_
#define TINY_H_

#define CR '\n'			// curse those different line endings
						// the final version will use 0x0a
#define TRACE 0			// set to 1 to show parser in progress
						// and computation stack trace
#define ASM 0			// disable compilation of assembly examples

#define BUFLEN 		80
#define MEM_START 	0x300
#define MEM_END		0x7fff	// set for top of available ram
						
// what sort of errors might we see?
typedef enum {
	ERR_NONE,
	ERR_MEM,
	ERR_SYNTAX,
	ERR_DIV0,
} ERROR_TYPE;
	

// the keywords are replaced by uint8_t with the high bit set
typedef enum {
	LET = 0x80,	// assign a value to a variable
	REM,		// a comment to ignore
	PRINT,		// print the following expression
	LIST,		// list the stored program
	NEW,		// delete the program in memory
	RUN,		// execute the program in memory
	INPUT,		// input a variable from the kb
	IF,			// test following expression, then do everything upto ENDIF
	ENDIF,		// otherwise, jump to line following this
	FOR,		// loop across a variable incrementing it by default
	TO,			// the end value
	NEXT,		// end of loop; go back (note: loop may be executed zero times!)
	DO,			// do the following instructions
	WHILE,		// while this condition is true
	GOTO,		// considered harmful, but unavoidable: go to line number
	GOSUB,		// call a routine at expression
	RETURN,		// return from the called routine
	END,		// end the program
	EQUAL,		// tests and assignments for expression parsing
	NEQUAL,
	GTE,
	GT,
	LTE,
	LT,
	LAST_KW,	// the end
} KEYWORD_ENUMS;

/* globals */
static char Look;			// the next character to consider
static ERROR_TYPE err;		// any error we might have
static int16_t the_line;	// the line we're currently executing

/* prototypes */

uint16_t squish_buffer (void);
void store_line (uint16_t this_line);
char * do_for (char * where);
char * execute (char * where);
uint8_t * find_next_line (uint8_t * this_line_add);
uint8_t * find_pair(char *where, KEYWORD_ENUMS pair);

void GetChar (void);
void SkipWhite (void);
void Error (char * s);
void Abort (char * s);
void Expected (char * s);
void Match (char x);
char GetName (void);
int16_t GetNum (void);
bool IsAddOp (char c);
bool IsMulOp (char c);
bool IsLogOp (char c);
void Emit (char * s);
void EmitLn (char * s);
int16_t Factor (void);
int16_t Term (void);
int16_t Expression (void);
uint8_t *  Assignment (uint8_t *);



#endif
