/* Preliminary tiny basic in C for proof of concepts
 * restricted to 26 signed integer variables A-Z and a single string $
 * (c) 2024 nailed_barnacle
 */
 
#include <stdio.h>
#include <stdint.h>
#include <ctype.h>
#include <string.h>
#include <stdbool.h>

#include "tiny.h"


// the keyword tokens, must be in same order as enums!
char tokens[][LAST_KW] = 
{
	"let",		// 0x80
	"'",		// 0x81
	"print",	// 0x82
	"list",		// 0x83
	"new",		// 0x84
	"run",		// 0x85
	"input",	// 0x86
	"if",		// 0x87
	"endif",	// 0x88
	"for",		// 0x89
	"to",		// 0x8a
	"next",		// 0x8b
	"do",		// 0x8c
	"while",	// 0x8d
	"goto",		// 0x8e
	"gosub",	// 0x8f
	"return",	// 0x90
	"end",		// 0x91
	"=",		// 0x92
	"!=",		// 0x93
	">=",		// 0x94
	">",		// 0x95
	"<=",		// 0x96
	"<",		// 0x97
};


char mem[32768];		// somewhere for the program to live
char buffer[BUFLEN];	// place for the input line
int16_t vars[26];		// the integer variables
char string[256] = "";	// the string variable $

int16_t buf_posn;		// where have we got to analysing the line?

char *	mem_next;
char *	mem_last;
char *	mem_ptr;		

void buf2hex (void)
{
	// show buffer as hex dump
	for (int r = 0; r < 5 ; r++)
	{
		for (int q = 0; q < 16; q++)
		{
			printf ("%02x ", (uint8_t)buffer[(r*16) + q]);
		}
		printf (" ");
		for (int q = 0; q < 16; q++)
		{
			char tmp = (uint8_t)buffer[(r*16) + q];
			if ((tmp < ' ') || (tmp > 0x7e))
				printf (".");
			else
				printf ("%c", tmp);
		}
		printf ("\n");
	}
}

void mem2hex (void)
{
	// show start of memory as hex dump
	printf ("mem starts at %p\n", mem + MEM_START);
	for (int r = 0; r < 8; r++)
	{
		for (int q = 0; q < 16; q++)
		{
			printf ("%02x ", (uint8_t)mem[(r*16) + q + MEM_START]);
		}
		printf (" ");
		for (int q = 0; q < 16; q++)
		{
			char tmp = (uint8_t)mem[(r*16) + q + MEM_START];
			if ((tmp < ' ') || (tmp > 0x7e))
				printf (".");
			else
				printf ("%c", tmp);
		}
		printf ("\n");
	}
}

uint16_t get_line (char * where)
{
	/* translate a pointer to a line into that line's number
	 */
	if (NULL == where)
		return 0;
	else
		return *(uint16_t *)where;
}

uint8_t count_to_cr (char * ptr)
{
	// count the number of characters between ptr and the next \n
	// including the \n
	int ret = 0;
	while ((CR != *ptr) && ('\0' != *ptr))
	{
		ptr++;
		ret++;
	}
	return ret + 1;
}

char * find_line (uint16_t the_line)
{
	/* seek the requested line, and return a pointer to either
	 * it, the first line found which is higher, or to the end
	 * of the memory (i.e. when the new line being entered is
	 * higher than any already entered)
	 * 
	 * line are stored with two bytes of line number and one
	 * byte of length, then the tokenised text
	 */

	char * where = &mem[MEM_START];
	uint16_t found;
	uint8_t length;
	
	while (where < &mem[MEM_END])
	{
		// what line number is here?
		found = * (uint16_t *) where;
		if (((uint16_t)found >= (uint16_t)the_line) || (0 == found))
			break;
		// else advance to the next line number
		where = find_next_line(where);
	}
	return where;
}

uint8_t * find_next_line (uint8_t * this_line_add)
{
	/* Enter with this_line_add pointing to the current line; 
	 * return the address of the next line, which might be zero for
	 * the end of the buffer
	 */
	
	uint8_t length;
	length = * (uint8_t *)(this_line_add + 2); 
	this_line_add = &this_line_add[length];
	return this_line_add;
}
	
uint16_t squish_buffer (void)
{
	/* given a line of text in buffer, compress it by tokenising any 
	 * keywords, replacing the ascii line number with its int16_t 
	 * equivalent, and inserting a line length byte
	 */
	
	uint16_t	linenum = GetNum();
	bool 		quote = false;
	int 		size;
	int 		q;	
	
	// Look is first non-white character; mem_ptr points to next
	mem_ptr--;	
	// leave a space for line number and length
	memmove (&buffer[3], mem_ptr, count_to_cr(mem_ptr));
	* (uint16_t *)buffer = linenum;
 	// now scan for tokens and replace them... 
 	mem_ptr = &buffer[3];
	while ((*mem_ptr != CR) && (mem_ptr < &buffer[BUFLEN - 2]))
	{
		if ('"' == *mem_ptr)			// don't tokenise quoted strings
		{
			quote = !quote;				// toggle the quote flag
		}
		if (!quote)
		{	
			// so tokenise it		
			for (q = LET; q < LAST_KW; q++)
			{
				size = strlen(tokens[q - 0x80]);
				if (0 == strncasecmp(tokens[q - 0x80], mem_ptr, size))
				{
					// it matched, eat token but leave space for its value
					memmove(&mem_ptr[1], 
							&mem_ptr[size], 
							strlen(&mem_ptr[size]));
					*mem_ptr = q;			// insert token value
					break;
				}
			}
			if (q == LAST_KW)
			{
				// if it didn't match, just move to the next character
				mem_ptr++;
			}
		}
		else
		{
			mem_ptr++;		// next character
		}
	}
	// save the new length of the compressed line and metadata
	buffer[2] = count_to_cr(&buffer[3]) + 3;
	return linenum;
}

void store_line (uint16_t this_line)
{
	/* store_line()
	 * The line has been compressed in place in buffer
	 * If the line is not in the store, insert it in its correct sequence
	 * If the line is in the store, replace it (or if enpty, delete it)
	 */

	uint16_t line_found;
	bool line_exists = false;
	int insert_length = 0;
	int existing_length = 0;
	int space;
		
	// seek this line in memory
	mem_ptr = find_line(this_line);

	line_found = * (uint16_t *) mem_ptr;		
	insert_length = * (uint8_t *) &buffer[2];
	space = &mem[MEM_END] - mem_last;
	printf ("%d free\n", space);
	
	if ((0 == line_found) && (insert_length > 4))	// empty line has length 4
	{
		// we point at the next available storage memory
		if (space > insert_length)
		{
			memmove (mem_ptr, buffer, insert_length);
			mem_ptr += insert_length;
			mem_last = mem_ptr;
		}
		else
		{
			err = ERR_MEM;
		}
	}	
	else if (line_found != 0)
	{
		if (line_found > this_line)
		{
			// we need to shuffle up and insert
			if (insert_length > 4)
			{
				// the line is valid, it needs space
				if (space > insert_length)
				{
					memmove (mem_ptr + insert_length, mem_ptr, mem_last - mem_ptr);
					mem_last += insert_length;
					memmove (mem_ptr, buffer, insert_length);
				}
				else
				{
					err = ERR_MEM;
				}
			}
			// we don't insert an empty line!
		}
		else
		{
			// we need to replace or delete this existing line
			existing_length = (* (uint8_t *)(mem_ptr + 2));
			if (insert_length > 4)
			{
				if (space > insert_length)
				{	
					// replace the line
					// probably need to move existing memory to fit the space
					memmove(mem_ptr + insert_length, mem_ptr + existing_length, mem_last - mem_ptr - existing_length);
					memmove(mem_ptr, buffer, insert_length);
					mem_last += insert_length;
					mem_last -= existing_length;
				}
				else
				{
					err = ERR_MEM;
				}
			}
			else
			{
				// it's empty; delete the line
				memmove(mem_ptr, mem_ptr + existing_length, mem_last - mem_ptr - existing_length);
				mem_last -= existing_length;
			}
		}
	}
	*(int16_t *) mem_last = 0;		// mark end of memory
}

void list (void)
{
	/* list either:
	 * all the program (when no parameters given)
	 * a single line (a line number is specified)
	 * a range (a starting and ending number are specified)
	 */
	 
	char *	 	where;
	char * 		next_line;
	uint16_t	first_line = 0;
	uint16_t	last_line = 32767;
	uint16_t 	found;
	uint8_t		ch;
	int8_t		indent = 0;
	
	// see if there are any parameters
	GetChar();
	if (CR != Look)
	{
		// ok, there's something after the list instruction
		SkipWhite();
		if (isdigit (Look))
		{
			// first parameter
			first_line = GetNum();
			last_line = first_line;	// let's assume it's just one line
			SkipWhite();
			if ('-' == Look)
			{
				// try for second parameter
				GetChar();
				SkipWhite();
				if (isdigit (Look))
				{
					last_line = GetNum();
				}
			}
		}
	}
	// we have established a range to list...
	where = find_line(first_line);	// so get its address
	found = * (uint16_t *) where;	// and read the number from memory
	while ((found > 0) && (found <= last_line))
	{
		// the line number
		printf ("%5d ", found);
		next_line = where + *(uint8_t *)(where + 2);
		where += 3;					// move to payload
		ch = * where;
		for (int q = 0; q < indent; q++)
		{
			// print the indent, if there is one
			printf ("  ");
		}
		// see if we want to indent after this line
		if ((ch == IF) || (ch == FOR) || (ch == DO))
		{
			indent++;
		}
		// or outdent
		if ((ch == ENDIF) || (ch == NEXT) || (ch == WHILE))
		{
			indent--;
			if (indent < 0)
			{
				indent = 0;
			}
		}
		while (ch != CR)
		{
			if (ch < 0x80)				// it's ascii, print it
			{
				printf ("%c", * where);
			}
			else						// it's a token
			{
				printf ("%s",tokens[ch - 0x80]);
			}
			where ++;
			ch = * where;
		}
		printf ("%c",CR);
		where = next_line;
		found = * (uint16_t *) where;
	}
}

uint8_t * print (uint8_t * where)
{
	/* The print command
	 * Prints either a quoted string, an expression, or the string $
	 * Prints a cr unless terminated with a ';'
	 * On entry, Look contains PRINT and where points immediately
	 * afterwards.
	 * returns a pointer to the next line if executed from memory
	 */

	bool newline;
	
	mem_ptr = where;
	// where is pointing to the statement, not the line number...
	where -= 4;
	GetChar();
	do
	{
		newline = true;
		SkipWhite();
		if (CR == Look)
		{
			// we do nothing here but do need to detect it
		}
		else if ('$' == Look)
		{
			GetChar();
			if ('[' == Look)	// we're asking for an index
			{
				Match ('[');
				printf ("%c", string[Expression()]);
				Match (']');
				
			}
			else	
			{
				printf ("%s", string);
			}
		}
		else if ('"' == Look)
		{
			GetChar();
			while (('"' != Look) && (CR != Look))
			{
				putchar(Look);
				GetChar();
			}
			Match ('"');
		}
		else if ((isalnum(Look)) || ('-' == Look) || ('%' == Look) || ('(' == Look))
		{
			if ('%' == Look)		// right justify, following space
			{
				GetChar();
				printf ("%6d ", Expression());
			}
			else 					// left justify, following space
			{
				printf ("%d ", Expression());
			}
		}
		else
		{
			err = ERR_SYNTAX;
			break;
		}
		if (';' == Look)
		{
			GetChar ();				// don't issue a following CR
			newline = false;
		}
		if (newline)
		{
			printf("\n");
		}
	}
	while ((CR != Look) && ('\0' != Look) && (ERR_NONE == err));
	
	return find_next_line (where);
}

void new (void)
{
	// delete all lines in memory by resetting pointers
	mem_last = &mem[MEM_START];
	mem_next = mem_last;
	*(int16_t *) mem_last = 0;		// mark end of memory
}

bool compare (void)
{
	/* decide whether _if condition is true
	 * we expect an expression, an comparator, and a second expression
	 * Note: uses mem_ptr 
	 */
	 
	uint32_t param1;
	uint32_t param2;
	KEYWORD_ENUMS comp;
	bool ret = false;
	
	GetChar();
	SkipWhite();
	param1 = Expression();
	SkipWhite();
	comp = (uint8_t)Look;
	if ((comp < EQUAL) || (comp > LT))
	{
		// we expected a comparator token but didn't see one
		err = ERR_SYNTAX;
	}
	GetChar();
	SkipWhite();
	param2 = Expression();
	if (EQUAL == comp)
	{
		ret = (param1 == param2);
	}
	else if (NEQUAL == comp)
	{
		ret = (param1 != param2);
	}
	else if (GTE == comp)
	{
		ret = (param1 >= param2);
	}
	else if (GT == comp)
	{
		ret = (param1 > param2);
	}
	else if (LTE == comp)
	{
		ret = (param1 <= param2);
	}
	else if (LT == comp)
	{
		ret = (param1 < param2);
	}
	return ret;
}

uint8_t * find_pair(char *where, KEYWORD_ENUMS pair)
{
	/* Locate a matching end-statement for if (endif), for (next), or
	 * do (while) taking account of nested calls
	 * Return NULL if no match found
	 */
	 
	uint16_t this_line;
	uint16_t depth = 0;
	KEYWORD_ENUMS match;
	
	if (IF == pair)
		match = ENDIF;
	else if (FOR == pair)
		match = NEXT;
	else if (DO == pair)
		match = WHILE;
	else return NULL;		// oops
	
	while (where != NULL)
	{
		this_line = * (uint16_t *)where;
		if (this_line != 0)
		{
			if (pair == (uint8_t)where[3])
			{
				depth ++;	// we're nesting
			}
			if (match == (uint8_t)where[3])
			{	
				if (0 == depth)
				{
					break;
				}
				else
				{
					depth --;
				}
			}
			where = find_next_line(where);
		}
		else
		{
			where = NULL;
		}
	}
	return where;
}

char * do_if (char * where)
{
	/* return the next line to execute based on the truth of the if
	 * statement
	 */

	where -= 4;				// back to the start of the line
	where = find_next_line(where);
	if (!compare())
	{
		where = find_pair(where, IF);
	}
	return where;
}

char * do_for (char * where)
{
	/* Handle for expression to expression command
	 * we set up the initial and ending conditions and then recursively
	 * call execute() until it sees a 'next' token, when it returns.
	 * Note: the step value is always one. However, the step counter can
	 * be manipulated in the loop - e.g. subtracting two to step by
	 * minus one.
	 */
	
	int16_t paramto;	// to
	char name;			// and which variable is the counter?
	uint16_t this_line;	// the line we start at
	char * this_where;	// the place we start at
	
	where -= 4;
	this_line = *(uint16_t *)where;
	where = find_next_line(where);
	this_where = where;
	
	GetChar();
	SkipWhite();
	if (!isalpha (Look))
	{
		// not a variable?
		where = NULL;
		//Expected("variable");
		err = ERR_SYNTAX;
	}
	else
	{
		// one of the 26 signed integer variables
		name = GetName ();
		SkipWhite();
		Match (EQUAL);
		name = name - 'A';
		vars[name] = Expression();	// the start value
		SkipWhite();
		Match (TO);
		paramto = Expression();		// the end value
		
		while ((vars[name] <= paramto) && (ERR_NONE == err))
		{
			// recursively execute lines until 'next'
			while (NULL != where)
			{
				where = execute(where);
				if (ERR_NONE != err)
					break;
			}	
			where = this_where;
			vars[name]++;			// normal step value
		}
		where = find_pair (this_where, FOR);
		where = find_next_line(where);
	}	

	return where;
}

char * do_do (char * where)
{
	/* Execute the 'do' clause by calling it recursively; it continues
	 * until the matching 'while' statement. That statement returns NULL
	 * if its comparison clause is true, indicating that the loop should
	 * continue, or the address of the line following 'while'
	 */
	
	char * loop_address;	// first line in the do loop
	char * cont_address;	// first line after the while
	char * while_address;	// the while line 
	
	where -= 4;
	where = find_next_line (where);
	loop_address = where;
	while_address = find_pair (where, DO);
	cont_address = find_next_line (while_address);	// first line after while
	do
	{
		where = loop_address;
		do
		{
			where = execute(where);
		}	
		while ((NULL != where) && (where != cont_address));
	}
	while ((NULL == where) && (ERR_NONE == err));
	return where;
}

char * do_while (char * where)
{
	/* test a condition; if true, return NULL 
	 * otherwise, return the address of the next line
	 */
	if (compare())
	{
		return NULL;
	}
	else
	{
		return find_next_line (where - 4);
	}
}

char * gosub (char * where)
{
	/* recursively execute instructions starting at Expression()
	 * so gosubs can be nested (and precalculated!)
	 */
	char * cont_address;
	cont_address = find_next_line(where - 4);
	
	GetChar();
	SkipWhite();
	where = find_line (Expression());
	do
	{
		where = execute(where);
	}	
	while ((NULL != where) && (ERR_NONE == err));
	return cont_address;
}

void input (void)
{
	/* input prints a '?' to the terminal, then waits for a user
	 * response. It evaluates this response as either an integer
	 * variable or a string 
	 */
	
	char * save_mem_ptr;
	char save_look;
	char name;
	bool minus = false;
	int16_t val;
	
	
	GetChar();
	SkipWhite();
	
	printf ("? ");
	fgets (buffer, 79, stdin);
	
	// are we reading to a variable (i.e. int) or the string?
	if (isalpha (Look))	
	{
		// one of the 26 signed integer variables
		name = GetName ();
		// but the expression comes from the buffer, not memory
		// save the current memory pointer
		save_mem_ptr = mem_ptr;
		save_look = Look;
		mem_ptr = buffer;
		GetChar();
		if ('-' == Look)
		{
			minus = true;
			GetChar();		// if we start with a -, note and move past
		}
		val = GetNum();
		if (minus)
		{
			val *= -1;
		}
		vars[name - 'A'] = val;
		// restore memory pointer
		mem_ptr = save_mem_ptr;
		Look = save_look;
	}
	else if ('$' == Look)
	{
		save_mem_ptr = mem_ptr;
		mem_ptr = buffer;
		name = 0;	// we recyle to same memory
		while (mem_ptr[name] != CR)
		{
			string[name] = mem_ptr[name];
			name++;
		}
		string[name] = '\0';
		mem_ptr = save_mem_ptr;
	}
	else
	{
		err = ERR_SYNTAX;
	}		
}

char * execute (char * where)
{
	/* Execute a single program line, pointed to by 'where'
	 * Return the next line to execute, or zero.
	 */
	
	the_line = get_line (where);

	if (the_line != 0)
	{
		mem_ptr = &where[3];			// first char on line
		GetChar();
		where = find_next_line (where);	// start of next line
	
		switch ((uint8_t)Look)
		{
			case LET:
				where = Assignment(mem_ptr); 
				break;
			case REM:
				// we do nothing here
				break;
			case PRINT:
				where = print(mem_ptr);
				break;
			case INPUT:
				input();
				break;
			case IF:
				where = do_if (mem_ptr);
				break;
			case ENDIF:
				// this is just a marker
				break;
			case FOR:
				where = do_for (mem_ptr);
				break;
			case NEXT:
				where = NULL;
				break;
			case DO:
				where = do_do (mem_ptr);
				break;
			case WHILE:
				where = do_while (mem_ptr);
				break;
			case GOTO:
				GetChar();
				SkipWhite();
				where = find_line (Expression());
				break;
			case GOSUB:
				where = gosub (mem_ptr);
				break;
			case RETURN:
				where = NULL;
				break;
			case END:
				where = NULL;
				break;
			default:
				if ((isalpha(Look)) || ('$' == Look))
				{
					where = Assignment(mem_ptr);
				}
				else
				{
					// anything else is an error
					err = ERR_SYNTAX;
				}
				break;
		}
	}
	return where;
}
	 
void run (void)
{
	/* Execute the program, one line at a time. Only one statement is
	 * permitted on each line.
	 * Each line executed returns a pointer to the next line to execute.
	 * When NULL is returned, we're done and return to main
	 */
	 
	char * where;
	
	where = &mem[MEM_START];
	do
	{
		the_line = get_line (where);
		where = execute(where);
	}
	while ((the_line != 0) && (ERR_NONE == err));
	// report an error
	if (ERR_NONE != err)
	{
		printf ("%d, What?\n", the_line);
	}
}

/***********************************************************************
 * expression evaluation routines
 * 
 * These attempt to evaluate an expression pointed to by mem_ptr
 * 
 * There are 26 int16_t variables labelled A-Z (case doesn't matter)
 * 
 * Arithmetic is restricted to signed 16-bit values.
 * 
 * Interpretation of expressions is derived from Dr Jack Crenshaw's work:
 * "Let's Build a Compiler"
 * 
 */
 

void GetChar (void)
{

	// get next character from buffer
	Look = *mem_ptr++;
}

void SkipWhite (void)
{
	// stops on end of string or CR
	while ((isspace(Look)) && ('\0' != Look) && (CR != Look))
	{
		GetChar();
	}
}

void Match (char x)
{
	// match a specific input character, get the next character
	// and skip over any following white space
	if (x == Look)
	{
		GetChar();
		SkipWhite();
	}
	else
	{
		// we expected a character we didn't get so a syntax error
		err = ERR_SYNTAX;
	}
}

char GetName (void)
{
	// validate and return a single character variable name
	char ret = ' ';
	if (!isalpha(Look))
	{
		// we expected a variable name, but didn't see one
		err = ERR_SYNTAX;
	}
	ret = toupper(Look);
	GetChar();
	SkipWhite();
	return ret;
}

int16_t GetNum (void)
{
	// get a number
	int16_t ret = 0;
	if (!isdigit(Look))
	{
		// we expected an integer, but didn't see one
		err = ERR_SYNTAX;
	}
	while (isdigit(Look))
	{
		ret *= 10;
		ret += Look - '0';
		GetChar();
	}
	SkipWhite();
	return ret;
}

bool IsAddOp (char c)
{
	return (('+' == c) || ('-' == c));
}

bool IsMulOp (char c)
{
	return (('*' == c) || ('/' == c));
}

int16_t Factor (void)
{
	int16_t factor;
	
	if (Look == '(')
	{
		Match ('(');
		factor = Expression ();
		Match (')');
	}
	else if (isalpha(Look))
	{
		factor = vars[GetName() - 'A'];
	}
	else
	{
		factor = GetNum();
	}
	return factor;
}
	
int16_t Term (void)
{
	int16_t term = Factor(); 
	
	if (ERR_NONE == err)
	{
		while (IsMulOp(Look))
		{
			// there are only two possibilities
			if ('*' == Look)
			{
				Match ('*');
				term *= Factor();
			}
			else 
			{
				int16_t tmp;
				Match ('/');
				tmp = Factor();
				if (tmp != 0)
					term /= tmp;
				else
				{
					term = 0;	// div0 error
					err = ERR_DIV0;
				}
			}
		}
	}
	return term;
}

int16_t Expression (void)
{
	int16_t value;

	if (IsAddOp(Look))
	{
		value = 0;
	}
	else
	{
		value = Term();
	}
	if (ERR_NONE == err)
	{
		while (IsAddOp(Look))
		{
			// there are only two possibilities
			if ('+' == Look)
			{
				Match ('+');
				value += Term();
			}
			else
			{
				Match ('-');
				value -= Term();
			}
		}
	}
	return value;
}

uint8_t * Assignment (uint8_t * where)
{
	
	mem_ptr = where;
	// where is pointing to the statement, not the line number...
	where -= 4;

	char name;
	// is the LET implied?
	if (!(isalpha(Look) || ('$' == Look)))
	{
		// in which case, we don't look for LET
		Match (LET);
	}
	SkipWhite();
	if (isalpha (Look))
	{
		// one of the 26 signed integer variables
		name = GetName ();
		SkipWhite();
		Match (EQUAL);
		vars[name - 'A'] = Expression();
	}
	else if ('$' == Look)
	{
		// the single string variable
		uint8_t ptr = 0;
		
		GetChar ();
		SkipWhite();
		Match (EQUAL);
		Match ('"');
		while (('"' != Look) && (CR != Look))
		{
			string[ptr++] = Look;
			GetChar ();
		}
		string[ptr] = '\0';
		Match ('"');
	}		
	return find_next_line (where);	
}
	
void Init (char * mem)
{
	/* initialise the globals to point to either program store or the
	 * buffer; clear the errors, and load the first character into Look
	 */ 
	mem_ptr = mem;
	err = ERR_NONE;
	SkipWhite();
	GetChar();
	SkipWhite();
}

int main (int argc, char ** argv)
{
	printf ("Neolithic Tiny Basic 65c02\n");
	
	// init stuff
	mem_last = &mem[MEM_START];
	mem_next = mem_last;
	* (uint16_t *)mem_last = 0;
	// variables are not initialised
	
	while (1)
	{
		/* we receive a line of text in the buffer.
		 * If it starts with a number, it's assumed to be a line to be 
		 * either entered into, or replacing, or deleting a line in the 
		 * memory and is tokenised and handled accordingly.
		 * 
		 * If it starts with ascii characters, then we assume it's a
		 * command to be handled immediately:
		 * 	"let" (or variable name) to set a variable
		 * 	"print" to output text or a variable
		 * 	"list" to output either all the text or a single line
		 * 	"new" to delete the existing program
		 *  "run" to execute the program starting at the lowest number
		 * 
		 * In this case, we tokenise the command and then execute it.
		 * 
		 * Otherwise, we have a 'what?' error.
		 */
		printf ("> ");
		fgets (buffer, 79, stdin);
		
		Init (buffer);		// this also clears the errors
		if (isdigit(Look))
		{
			// it's a line number
			uint16_t linenum = squish_buffer();
			if (linenum > 0)
			{
				store_line(linenum);
			}
			else
			{
				err = ERR_SYNTAX;
			}
		}
		else
		{
			// try and execute it...
			// cheat an ascii zero in to the start of the line
			memmove(buffer+2, buffer, BUFLEN - 2);
			buffer[0] = '0';
			buffer[1] = ' ';
			Look = '0';
			squish_buffer(); 			// tokenise the line
			the_line = 0;				// for error purposes
			mem_ptr = &buffer[3];
			GetChar();
			switch ((uint8_t)Look)		// the first token
			{
				case LET:
					Assignment(mem_ptr);
					break;
				case PRINT:
					print(mem_ptr);
					break;
				case LIST:
					list();
					break;
				case NEW:
					new();
					break;
				case RUN:
					run();
					break;
				default:
					// if it begins with an alpha character or $ perhaps
					// it's an implied 'let'?
					if ((isalpha(Look)) || ('$' == Look))
					{
						Assignment(mem_ptr);
					}
					else
					{
						// anything else is an error
						err = ERR_SYNTAX;
					}
					break;
			}
		}
		switch (err)
		{
			case ERR_DIV0:
				printf ("Div/0\n");
				break;
			case ERR_SYNTAX:
				printf ("What?\n");
				break;
			case ERR_MEM:
				printf ("Memory!\n");
				break;
			case ERR_NONE:
			default:
				break;
		}
	}
	return 0;
}
